/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.importer.RequestImportEditor");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.form.SimpleTextarea");

dojo.require("rmps.app.ui.internal.editors.BaseEditor");

dojo.require("rmps.importer.ImportStatusEditorInput");
dojo.require("rmps.importer.ImportClient");
dojo.require("rmps.importer.util");

dojo.require("jazz.app.ua.TextHoverHelp");

dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.importer", "RequestImportEditorMessages");

var messages = dojo.i18n.getLocalization("rmps.importer", "RequestImportEditorMessages");

var Util = rmps.importer.util;

var SNAPSHOT_NAME_UAKEY = "com.ibm.xtools.rmps.web.importer.text_RequestImportEditor001";

var connectOnInput = function(obj, widget, method) {
	if (dojo.isMoz || dojo.isOpera) {
		widget.connect(obj, "oninput", method);
	} else {
		widget.connect(obj, "onkeydown", method);
		widget.connect(obj, "onkeyup", method);
		widget.connect(obj, "onpaste", function() {
			setTimeout(dojo.hitch(this, method), 1);
		});
		widget.connect(obj, "oncut", function() {
			setTimeout(dojo.hitch(this, method), 1);
		});
		widget.connect(obj, "onblur", method);
	}
};

var createRequiredNode = function() {
	var reqNode = dojo.create("span");
	reqNode.className = "required";
	reqNode.innerHTML = "*";
	reqNode.title = messages.requiredTitle;
	return reqNode;
};

dojo.declare("rmps.importer.RequestImportEditor", rmps.app.ui.internal.editors.BaseEditor, {
	
	"class": "rmps-importer-request",
	
	// init properties
	isRefreshable: false,
	isReadOnly: true,
	
	_validate: false,
	_fullImport: false,
	_createBaseline: false,
	_baselineName: null,
	_baselineDescription: null,
	
	createContentWidget: function() {
		this._loadCookie();
		return new rmps.importer.RequestImportEditor._Content({editor: this});
	},
	
	addButtons: function(toolbar) {
		toolbar.addWidget(new rmps.ui.SimpleButton({
				label: messages.requestImport,
				onClick: dojo.hitch(this, this._requestImport)
			}));
	},
	
	_requestImport: function() {
		if (this._createBaseline && !this._baselineName) {
			//baseline name is required
			this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: messages.error_request_no_baseline_name,
					closable: true
				});
		} else {
			this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
					summary: messages.requestingImport
				});
		
			if (this._createBaseline) {
				var baselineName = this._baselineName;
				var baselineDescription = this._baselineDescription;
			}
			var validate = this._validate;
			var deferred = rmps.importer.ImportClient.requestImport(this.getEditorInput().projectId, this._validate, this._fullImport, baselineName, baselineDescription);
			deferred.addCallback(this, function() {
				this._refreshStatus();
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.OK,
					summary: dojo.string.substitute(
						validate ? messages.importRequestSuccessful_validation : messages.importRequestSuccessful,
						[Util.getImportStatusUri()]),
					closable: true
				});
			});
			deferred.addErrback(this, function(error) {
				if (error.status === 412) {
					var message = dojo.string.substitute(messages.error_request_no_definition, [Util.getViewImportDefinitionUri()]);
				} else if (error.status === 403) {
					var message = messages.error_request_permission;
				} else if (error.status === 409 && this._createBaseline) { 
					var message = dojo.string.substitute(messages.error_request_baselineNameConflict, [this._baselineName]);
				} else {
					var message = messages.error_request_general;
				}
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: message,
					closable: true
				});
			});
		}
	},
	
	_refreshStatus: function() {
		var editorUtil = rmps.app.ui.EditorUtil.getInstance(this.domNode);
		var statusEditor = editorUtil.getEditor(
				new rmps.importer.ImportStatusEditorInput({projectId: this.getEditorInput().projectId}));
		if (statusEditor) {
			statusEditor.refresh();
		}
	},
	
	_loadCookie: function() {
		var value = dojo.cookie(this.declaredClass);
		if (value) {
			var cookie = dojo.fromJson(value);
			this._validate = !!cookie.validate;
			this._fullImport = !!cookie.fullImport;
			this._createBaseline = !!cookie.createBaseline;
		}
	},
	
	_saveCookie: function() {
		var value = dojo.cookie(this.declaredClass);
		if (value) {
			var cookie = dojo.fromJson(value);
		} else {
			cookie = {};
		}
		cookie.validate = this._validate;
		cookie.fullImport = this._fullImport;
		//do not save the baseline name and description in the cookie
		cookie.createBaseline = this._createBaseline;
		dojo.cookie(this.declaredClass, dojo.toJson(cookie), {expires: 90});
	}
});

dojo.declare("rmps.importer.RequestImportEditor._Content", [dijit._Widget, dijit._Templated], {
	
	messages: messages, 
	
	widgetsInTemplate: true,
	templateString: '<div>' +
				'<div class="item">' +
					'<input dojoType="dijit.form.CheckBox" dojoAttachPoint="vCb" dojoAttachEvent="onClick: _onValidate"/>' +
					'<label>${messages.validate}</label>' +
				'</div>' +
				'<div class="item">' +
					'<input dojoType="dijit.form.CheckBox" dojoAttachPoint="riCb" dojoAttachEvent="onClick: _onReImport"/>' +
					'<label>${messages.fullImport}</label>' +
					'<div class="fullImport-warning" dojoAttachPoint="_fullImportWarning" style="display: none;">${messages.fullImport_warning}</div>' +
				'</div>' +
				'<div class="item">' +
					'<input dojoType="dijit.form.CheckBox" dojoAttachPoint="bCb" dojoAttachEvent="onClick: _onBaseline"/>' +
					'<label>${messages.createBaseline}</label>' +
				'</div>' +
				'<fieldset style="margin-left:20px;" class="baseline-details">' +
					'<legend>Snapshot Details</legend>' +
					'<table>' +
						'<tr>' +
							'<td style="width:1px;"><span dojoAttachPoint="_baselineNameLabel">${messages.baselineName}</span></td>' +
							'<td style="width:100%;"><input type="text" dojoAttachPoint="_baselineName"/></td>' +
						'</tr>' +
						'<tr>' +
							'<td>${messages.baselineDescription}</td>' +
							'<td style="width:100%;"><textarea rows="4" dojoAttachPoint="_baselineDescription" dojoAttachEvent="onKeyPress: _onDescriptionChange"></textArea></td>' +
						'</tr>' +
					'</table>' +
				'</fieldset>' +				
			'</div>',
	
	// init properties
	editor: null,
	
	_help: null,
	
	postCreate: function() {
		this.vCb.attr("value", this.editor._validate);
		this.riCb.attr("value", this.editor._fullImport);
		this.bCb.attr("value", this.editor._createBaseline);
		this.vCb.domNode.nextSibling.setAttribute("for", this.vCb.id);
		this.riCb.domNode.nextSibling.setAttribute("for", this.riCb.id);
		this.bCb.domNode.nextSibling.setAttribute("for", this.bCb.id);
		this._fullImportWarning.style.display = this.editor._fullImport ? "" : "none";
		this._baselineName.disabled = !this.editor._createBaseline;
		//add a "required" star to the name label
		this._baselineNameLabel.parentNode.appendChild(createRequiredNode());
		this._baselineDescription.disabled = !this.editor._createBaseline;
		
		this._help = [];
		this._help.push(new jazz.app.ua.TextHoverHelp({uaKey: SNAPSHOT_NAME_UAKEY, contentNode: this._baselineNameLabel}));
		
		connectOnInput(this._baselineName, this, this._onNameChange);
		connectOnInput(this._baselineDescription, this, this._onDescriptionChange);
	},
	
	_onValidate: function() {
		this.editor._validate = !!this.vCb.attr("value");
		this.editor._saveCookie();
	},
	
	_onReImport: function() {
		this.editor._fullImport = !!this.riCb.attr("value");
		this._fullImportWarning.style.display = this.editor._fullImport ? "" : "none";
		this.editor._saveCookie();
	},
	
	_onBaseline: function() {
		this.editor._createBaseline = !!this.bCb.attr("value");
		this._baselineName.disabled = !this.editor._createBaseline;
		this._baselineDescription.disabled = !this.editor._createBaseline;
		this.editor._saveCookie();
	},
	
	_onNameChange: function() {
		this.editor._baselineName = this._baselineName.value;
	},
	
	_onDescriptionChange: function() {
		this.editor._baselineDescription = this._baselineDescription.value;
	},
	
	startup: function() {
		this.inherited(arguments);	

		dojo.forEach(this._help, function(help) {
			help.startup();
		});
	},
	
});
})();
